@@ -128,6 +128,7 @@ sub entropy_bytes {
 *is_carmichael = \&Math::Prime::Util::PP::is_carmichael;
 *is_quasi_carmichael = \&Math::Prime::Util::PP::is_quasi_carmichael;
 *is_pillai = \&Math::Prime::Util::PP::is_pillai;
+*is_fundamental = \&Math::Prime::Util::PP::is_fundamental;
 
 *random_prime = \&Math::Prime::Util::PP::random_prime;
 *random_ndigit_prime = \&Math::Prime::Util::PP::random_ndigit_prime;
@@ -486,6 +487,13 @@ sub factorial {
   return Math::Prime::Util::PP::factorial($n);
 }
 
+sub factorialmod {
+  my($n, $m) = @_;
+  _validate_integer($n);
+  _validate_integer($m);
+  return Math::Prime::Util::PP::factorialmod($n, $m);
+}
+
 sub binomial {
   my($n, $k) = @_;
   _validate_integer($n);
@@ -772,7 +780,8 @@ sub Pi {
 my $_exitloop = 0;
 sub lastfor { $_exitloop = 1; }
 sub _get_forexit { $_exitloop; }
-sub _set_forexit { $_exitloop = $_[0] ? 1 : 0; }
+sub _start_for_loop { my $old = $_exitloop; $_exitloop = 0; $old; }
+sub _end_for_loop { $_exitloop = shift; }
 
 sub forprimes (&$;$) {    ## no critic qw(ProhibitSubroutinePrototypes)
   my($sub, $beg, $end) = @_;
@@ -780,7 +789,7 @@ sub forprimes (&$;$) {    ## no critic qw(ProhibitSubroutinePrototypes)
   _validate_num($beg) || _validate_positive_integer($beg);
   _validate_num($end) || _validate_positive_integer($end);
   $beg = 2 if $beg < 2;
-  my $oldexitloop = $_exitloop;  $_exitloop = 0;
+  my $oldforexit = _start_for_loop();
   {
     my $pp;
     local *_ = \$pp;
@@ -790,7 +799,7 @@ sub forprimes (&$;$) {    ## no critic qw(ProhibitSubroutinePrototypes)
       last if $_exitloop;
     }
   }
-  $_exitloop = $oldexitloop;
+  _end_for_loop($oldforexit);
 }
 
 sub forcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
@@ -800,7 +809,7 @@ sub forcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
   _validate_num($end) || _validate_positive_integer($end);
   $beg = 4 if $beg < 4;
   $end = Math::BigInt->new(''.~0) if ref($end) ne 'Math::BigInt' && $end == ~0;
-  my $oldexitloop = $_exitloop;  $_exitloop = 0;
+  my $oldforexit = _start_for_loop();
   {
     my $pp;
     local *_ = \$pp;
@@ -812,7 +821,7 @@ sub forcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
       }
     }
   }
-  $_exitloop = $oldexitloop;
+  _end_for_loop($oldforexit);
 }
 
 sub foroddcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
@@ -823,7 +832,7 @@ sub foroddcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
   $beg = 9 if $beg < 9;
   $beg++ unless $beg & 1;
   $end = Math::BigInt->new(''.~0) if ref($end) ne 'Math::BigInt' && $end == ~0;
-  my $oldexitloop = $_exitloop;  $_exitloop = 0;
+  my $oldforexit = _start_for_loop();
   {
     my $pp;
     local *_ = \$pp;
@@ -835,14 +844,14 @@ sub foroddcomposites(&$;$) { ## no critic qw(ProhibitSubroutinePrototypes)
       }
     }
   }
-  $_exitloop = $oldexitloop;
+  _end_for_loop($oldforexit);
 }
 
 sub fordivisors (&$) {    ## no critic qw(ProhibitSubroutinePrototypes)
   my($sub, $n) = @_;
   _validate_num($n) || _validate_positive_integer($n);
   my @divisors = divisors($n);
-  my $oldexitloop = $_exitloop;  $_exitloop = 0;
+  my $oldforexit = _start_for_loop();
   {
     my $pp;
     local *_ = \$pp;
@@ -852,7 +861,7 @@ sub fordivisors (&$) {    ## no critic qw(ProhibitSubroutinePrototypes)
       last if $_exitloop;
     }
   }
-  $_exitloop = $oldexitloop;
+  _end_for_loop($oldforexit);
 }
 
 sub forpart (&$;$) {    ## no critic qw(ProhibitSubroutinePrototypes)
