@@ -30,9 +30,9 @@ static const uint32_t small_ram_upper_idx[] = {
   64439,65566,65829,84761,89055,104959,107852,146968,151755,186499,217258,
   223956,270700,332195,347223,440804,508096,565039,768276,828377,1090285,
   1277320,1568165,1896508,2375799,3300765,4162908,5124977,6522443,9298256,
-  11406250, 15528199, 20637716, 28239295, 39711166,
-  55623925, 80161468, 117683515, 174200145, 261514813,
-  404761877, 633278258, 1024431762, 1683645810, UVCONST(2868095461),
+  11406250, 15873245, 21307556, 29899174, 40666215,
+  57180770, 81543888, 119596564, 177392936, 266391665,
+  411512446, 646331578, 1043239835, 1723380058, UVCONST(2919198776),
   UVCONST(4294967295)
 };
 #define SMALL_NRAM_UPPER_MULT 2852
@@ -40,40 +40,40 @@ static const uint32_t small_ram_upper_idx[] = {
 
 #if BITS_PER_WORD == 64
 static const UV large_ram_upper_idx[] = {
-  UVCONST(     2209850625), UVCONST(     2505047527), UVCONST(     2868095461),
+  UVCONST(     2256197513), UVCONST(     2556868249), UVCONST(     2919198776),
   /* 11071, 11070, 11069,   11068, 11067, 11066,   11065, 11064, 11063 */
-  UVCONST(     3300302120), UVCONST(     3790618144), UVCONST(     4350498841),
-  UVCONST(     5046044184), UVCONST(     5822787413), UVCONST(     6745591770),
-  UVCONST(     7843974826), UVCONST(     9136430799), UVCONST(    10622435139),
+  UVCONST(     3371836636), UVCONST(     3874119737), UVCONST(     4467380631),
+  UVCONST(     5163817509), UVCONST(     5950413657), UVCONST(     6901033442),
+  UVCONST(     8015893438), UVCONST(     9322299866), UVCONST(    10845166831),
   /* 11062, 11061, 11060,   11059, 11058, 11057,   11056, 11055, 11054 */
-  UVCONST(    12458964476), UVCONST(    14547850676), UVCONST(    17105209669),
-  UVCONST(    20137461647), UVCONST(    23754200841), UVCONST(    28113266506),
-  UVCONST(    33244053524), UVCONST(    39635847319), UVCONST(    47216781912),
+  UVCONST(    12727569836), UVCONST(    14852585181), UVCONST(    17463419944),
+  UVCONST(    20585027534), UVCONST(    24252210453), UVCONST(    28704897522),
+  UVCONST(    34003499133), UVCONST(    40436019651), UVCONST(    48229247660),
   /* 11053, 11052, 11051,   11050, 11049, 11048,   11047, 11046, 11045 */
-  UVCONST(    56400647995), UVCONST(    67708204893), UVCONST(    81253508908),
-  UVCONST(    98059403896), UVCONST(   118606509360), UVCONST(   143852101796),
-  UVCONST(   175186371899), UVCONST(   213694229671), UVCONST(   261738209720),
+  UVCONST(    57558675911), UVCONST(    69028965312), UVCONST(    83015434548),
+  UVCONST(   100138535684), UVCONST(   121051505524), UVCONST(   146783829698),
+  UVCONST(   178727808587), UVCONST(   218113299173), UVCONST(   267104085772),
   /* 11044, 11043, 11042,   11041, 11040, 11039,   11038, 11037, 11036 */
-  UVCONST(   321608703183), UVCONST(   396839992097), UVCONST(   490698410355),
-  UVCONST(   609608518202), UVCONST(   760145301247), UVCONST(   951394830802),
-  UVCONST(  1195260877070), UVCONST(  1507711305081), UVCONST(  1910188609050),
+  UVCONST(   328057281739), UVCONST(   404608665617), UVCONST(   500552556306),
+  UVCONST(   621794385742), UVCONST(   774739900202), UVCONST(   969943548548),
+  UVCONST(  1218276754392), UVCONST(  1536655221634), UVCONST(  1946308957195),
   /* 11035, 11034, 11033,   11032, 11031, 11030,   11029, 11028, 11027 */
-  UVCONST(  2429318332761), UVCONST(  3104355953395), UVCONST(  3983636468774),
-  UVCONST(  5136852322734), UVCONST(  6654037637976), UVCONST(  8663089898701),
-  UVCONST( 11333068937712), UVCONST( 14903446657241), UVCONST( 19702880416316),
+  UVCONST(  2475456777850), UVCONST(  3162491651655), UVCONST(  4058282334559),
+  UVCONST(  5233096936468), UVCONST(  6776539822896), UVCONST(  8821085181511),
+  UVCONST( 11539712635284), UVCONST( 15171808426849), UVCONST( 20056581407599),
   /* 11026, 11025, 11024,   11023, 11022, 11021,   11020, 11019, 11018 */
-  UVCONST( 26192198741217), UVCONST( 35007464587700), UVCONST( 47072080175048),
-  UVCONST( 63662369320719), UVCONST( 86631548367209), UVCONST(118640390543125),
-  UVCONST(163548371026622), UVCONST(226944845224518), UVCONST(317123850443715),
+  UVCONST( 26656864542121), UVCONST( 35627338984775), UVCONST( 47899755943330),
+  UVCONST( 64773009691258), UVCONST( 88134778026475), UVCONST(120680838280663),
+  UVCONST(166331208358410), UVCONST(230783974844445), UVCONST(322443487572932),
   /* 11017, 11016, 11015,   11014, 11013, 11012,   11011, 11010, 11009 */
-  UVCONST(446332665856000), /* estimates here and further */
-  UVCONST(632810110373375),
-  UVCONST(906250000000000),
-  1.50*UVCONST(906250000000000),
-  2.25*UVCONST(906250000000000),
-  3.40*UVCONST(906250000000000),
-  5.10*UVCONST(906250000000000),
-  7.60*UVCONST(906250000000000),
+  UVCONST(453738479744216), UVCONST(643248344602940), UVCONST(918867804392140),
+  UVCONST(1322953724888193),UVCONST(1920282116080684),
+  1.47*UVCONST(1920282116080684),  /* Estimates for larger */
+  2.3*UVCONST(1920282116080684),
+  3.4*UVCONST(1920282116080684),
+  5.1*UVCONST(1920282116080684),
+  7.9*UVCONST(1920282116080684),
+  12.2*UVCONST(1920282116080684),
 };
 #define LARGE_NRAM_UPPER_MULT 11075
 #define LARGE_NRAM_UPPER (sizeof(large_ram_upper_idx)/sizeof(large_ram_upper_idx[0]))
@@ -85,7 +85,7 @@ UV nth_ramanujan_prime_upper(UV n) {
   if (n <= 2) return (n==0) ? 0 : (n==1) ? 2 : 11;
   res = nth_prime_upper(3*n);
 
-  if (n < UVCONST(2209850625) || BITS_PER_WORD < 64) {
+  if (n < UVCONST(2256197512) || BITS_PER_WORD < 64) {
     /* While p_3n is a complete upper bound, Rp_n tends to p_2n, and
      * SNN(2011) theorem 4 shows how we can find (m,c) values where m < 1,
      * Rn < m*p_3n for all n > c.  Here we use various quantized m values
@@ -112,49 +112,55 @@ UV nth_ramanujan_prime_upper(UV n) {
     else                     res = (res * mult) >> 14;
 #endif
   }
+  if (n > 43 && n < 10000) {
+    /* Calculate upper bound using Srinivasan and Arés 2017 */
+    /* TODO We should construct a tighter bound like this. */
+    double s = (2 * (double)n) * (1.0L + 1.0L/ramanujan_sa_gn(n));
+    UV ps = nth_prime_upper( (UV) s );
+    if (ps < res)
+      res = ps;
+  }
   return res;
 }
 static const uint32_t small_ram_lower_idx[] = {
   2786, 2801, 4275, 5935, 6107, 8797, 9556, 13314, 13641, 20457, 23745,
   34432, 50564, 69194, 97434, 149399, 224590, 337116, 514260, 804041,
-  1367781, 2448102, 4513600, 8581572, 17555268, 39295429, 96043141,
-  255707435, 771748027, UVCONST(2691176863), UVCONST(4294967295)
+  1317612, 2340461, 4332796, 8393680, 17227225, 38996663, 94437897,
+  253560792, 763315838, UVCONST(2663598260), UVCONST(4294967295)
 };
 #define SMALL_NRAM_LOWER_MULT 557
 #define SMALL_NRAM_LOWER (sizeof(small_ram_lower_idx)/sizeof(small_ram_lower_idx[0]))
 
 #if BITS_PER_WORD == 64
 static const UV large_ram_lower_idx[] = {
-  UVCONST(    2287326511), UVCONST(    2691176863), UVCONST(    3186734059),
-  UVCONST(    3773549435), UVCONST(    4490874244), UVCONST(    5351635422),
-  UVCONST(    6379440261), UVCONST(    7669615016), UVCONST(    9222420352),
-  UVCONST(   11116948392), UVCONST(   13468245052), UVCONST(   16342357141),
+  UVCONST(    2267483962), UVCONST(    2663598260), UVCONST(    3152476871),
+  UVCONST(    3742932857), UVCONST(    4446913643), UVCONST(    5298293978),
+  UVCONST(    6318053149), UVCONST(    7608807497), UVCONST(    9140758346),
+  UVCONST(   11015956390), UVCONST(   13351265915), UVCONST(   16199147294),
   /* 4213, 4212, 4211,   4210, 4209, 4208,   4207, 4206, 4205 */
-  UVCONST(   19922979418), UVCONST(   24348237409), UVCONST(   29901283153),
-  UVCONST(   36755324577), UVCONST(   45489957420), UVCONST(   56422464192),
-  UVCONST(   70346667919), UVCONST(   87960867006), UVCONST(  110516423444),
+  UVCONST(   19739499402), UVCONST(   24137542585), UVCONST(   29629560254),
+  UVCONST(   36435870727), UVCONST(   45085624406), UVCONST(   55940244390),
+  UVCONST(   69713814138), UVCONST(   87221199999), UVCONST(  109606558728),
   /* 4204, 4203, 4202,   4201, 4200, 4199,   4198, 4197, 4196 */
-  UVCONST(  139376328492), UVCONST(  176769212370), UVCONST(  224987344210),
-  UVCONST(  287637947625), UVCONST(  369783640662), UVCONST(  477404340380),
-  UVCONST(  619830096553), UVCONST(  808881647349), UVCONST( 1061320360069),
+  UVCONST(  138227790751), UVCONST(  175290761423), UVCONST(  223132516788),
+  UVCONST(  285315117360), UVCONST(  366761235749), UVCONST(  473606049986),
+  UVCONST(  614858505562), UVCONST(  802552362351), UVCONST( 1052957884730),
   /* 4195, 4194, 4193,   4192, 4191, 4190,   4189, 4188, 4187 */
-  UVCONST(  1400537557830), UVCONST(  1858188046854), UVCONST(  2480691884624),
-  UVCONST(  3332007446416), UVCONST(  4503079799802), UVCONST(  6126600987695),
-  UVCONST(  8391511555810), UVCONST( 11573178646156), UVCONST( 16075623921843),
+  UVCONST(  1389550174208), UVCONST(  1843854433659), UVCONST(  2461728402552),
+  UVCONST(  3306766457564), UVCONST(  4469341663210), UVCONST(  6080948095909),
+  UVCONST(  8329279118918), UVCONST( 11488848759561), UVCONST( 15959135388235),
   /* 4186, 4185, 4184,   4183, 4182, 4181,   4180, 4179, 4178 */
-  UVCONST( 22497437657452), UVCONST( 31726512746004), UVCONST( 45096957879914),
-  UVCONST( 64630470145902), UVCONST( 93416220154664), UVCONST(136212405571722),
-  UVCONST(200433884291811),
-  UVCONST(297735302623021),
-  UVCONST(446609489920000), /* estimates here and further */
+  UVCONST( 22336622435614), UVCONST( 31501671598985), UVCONST( 44779902229212),
+  UVCONST( 64180867011184), UVCONST( 92772523880955), UVCONST(135282253437392),
+  UVCONST(199079826917291), UVCONST(295746797998912), UVCONST(443667118326600),
   /* 4177, 4176, 4175,   4174, 4173, 4172,   4171, 4170, 4169 */
-  UVCONST(676524345920000),
-  UVCONST(1025000000000000),
-  1.50*UVCONST(1025000000000000),
-  2.25*UVCONST(1025000000000000),
-  3.37*UVCONST(1025000000000000),
-  5.06*UVCONST(1025000000000000),
-  7.59*UVCONST(1025000000000000),
+  UVCONST(672350086039900),UVCONST(1029719394152693),UVCONST(1594365662292999),
+  1.55*UVCONST(1594365662292999), /* estimates here and further */
+  2.45*UVCONST(1594365662292999),
+  3.90*UVCONST(1594365662292999),
+  6.30*UVCONST(1594365662292999),
+  10.4*UVCONST(1594365662292999),
+  17.2*UVCONST(1594365662292999),
 };
 #define LARGE_NRAM_LOWER_MULT 4225
 #define LARGE_NRAM_LOWER (sizeof(large_ram_lower_idx)/sizeof(large_ram_lower_idx[0]))
@@ -166,7 +172,7 @@ UV nth_ramanujan_prime_lower(UV n) {
 
   res = nth_prime_lower(2*n);
 
-  if (n < UVCONST(2287326511) || BITS_PER_WORD < 64) {
+  if (n < UVCONST(2267483962) || BITS_PER_WORD < 64) {
     for (i = 0; i < SMALL_NRAM_LOWER; i++)
       if (small_ram_lower_idx[i] > n)
         break;
